@@ -13,10 +13,14 @@
 #'   - Column 3~m: Confounders (C)
 #'
 #' @param w A vector of exposure level to compute CERF.
-#' @param GPS_m A data.frame of GPS vectors.
-#'   - Column 1: GPS
-#'   - Column 2: Prediction of exposure for covariate of each data sample (e_gps_pred).
-#'   - Column 3: Standard deviation of  e_gps (e_gps_std)
+#' @param gps_m An S3 gps object including:
+#'   gps: A data.frame of GPS vectors.
+#'     - Column 1: GPS
+#'     - Column 2: Prediction of exposure for covariate of each data sample
+#'     (e_gps_pred).
+#'     - Column 3: Standard deviation of  e_gps (e_gps_std)
+#'   used_params:
+#'     - dnorm_log: TRUE or FLASE
 #' @param params A list of parameters that is required to run the process.
 #' These parameters include:
 #'   - alpha: A scaling factor for the GPS value.
@@ -24,15 +28,12 @@
 #'   - g_sigma: A scaling factor for kernel function (gamma/sigma).
 #'   - tune_app: A tuning approach. Available approaches:
 #'     - all: try all combinations of hyperparameters.
-#'   - expand: Scaling factor to determine the total number of nearest neighbors.
-#'   The total is \code{2 * expand * n_neighbour}.
 #'   - n_neighbor: Number of nearest neighbors on one side.
 #'   - block_size: Number of samples included in a computation block. Mainly
 #'   used to balance the speed and memory requirement. Larger \code{block_size}
 #'   is faster, but requires more memory.
 #' alpha, beta, and g_sigma can be a vector of parameters.
 #' @param kernel_fn A kernel function. A default value is a Gaussian Kernel.
-#' @param formula A formula to indicate the design matrix of the model for GPS.
 #' @param nthread An integer value that represents the number of threads to be
 #' used by internal packages.
 #'
@@ -50,27 +51,25 @@
 #' set.seed(19)
 #' data <- generate_synthetic_data(sample_size = 120, gps_spec = 3)
 #' # Estimate GPS function
-#' GPS_m <- train_gps(cov_mt = data[,-(1:2)],
-#'                    w_all = data$treat,
-#'                    sl_lib = c("SL.xgboost"),
-#'                    dnorm_log = FALSE)
+#' gps_m <- estimate_gps(cov_mt = data[,-(1:2)],
+#'                       w_all = data$treat,
+#'                       sl_lib = c("SL.xgboost"),
+#'                       dnorm_log = FALSE)
 #' # exposure values
 #' w.all <- seq(0,20,2)
 #' cerf_nngp_obj <- estimate_cerf_nngp(data,
 #'                                     w.all,
-#'                                     GPS_m,
+#'                                     gps_m,
 #'                                     params = list(alpha = c(0.1),
 #'                                                   beta = 0.2,
 #'                                                   g_sigma = 1,
 #'                                                   tune_app = "all",
 #'                                                   n_neighbor = 20,
-#'                                                   expand = 1,
 #'                                                   block_size = 1e4),
-#'                                     formula = ~ . - 1 - Y - treat,
 #'                                     nthread = 1)
 #'}
 #'
-estimate_cerf_nngp <- function(data, w, GPS_m, params, formula,
+estimate_cerf_nngp <- function(data, w, gps_m, params,
                                kernel_fn = function(x) exp(-x ^ 2),
                                nthread = 1) {
 
@@ -84,10 +83,14 @@ estimate_cerf_nngp <- function(data, w, GPS_m, params, formula,
   logger::log_info("Working on estimating cerf using nngp approach ...")
 
   # Double-check input parameters ----------------------------------------------
+  if (any(is.na(data))) {
+    stop("At this time, data with missing values is not supported.")
+  }
+
   check_params <- function(my_param, params) {
     for (item in my_param) {
       if (!is.element(c(item), names(params))) {
-        stop(paste0("The required parameter, ", item,", is not provided. ",
+        stop(paste0("The required parameter, ", item, ", is not provided. ",
                     "Current parameters: ", paste(unlist(names(params)),
                                                   collapse = ", ")))
       }
@@ -95,16 +98,39 @@ estimate_cerf_nngp <- function(data, w, GPS_m, params, formula,
   }
 
   check_params(c("alpha", "beta", "g_sigma",
-                 "tune_app", "n_neighbor",
-                 "expand", "block_size"), params)
+                 "tune_app", "n_neighbor", "block_size"), params)
+
+  if (!inherits(gps_m, "gps")) {
+    stop(paste0("The gps_m should be a gps class. ",
+                "Current format: ", class(gps_m)[1]))
+  }
+
+  if (nrow(data) != length(gps_m$gps$w)) {
+    stop(paste0("Provided Data and GPS object should have the same size.",
+                "Current sizes: ", nrow(data), " vs ", length(gps_m$gps$w)))
+  }
+
 
   # TODO: Check values of parameters, too.
 
+  # Order data based on w ------------------------------------------------------
+  data <- data[order(data[, c(2)]), ]
+  gps_m$gps <- gps_m$gps[order(gps_m$gps$w), ]
+
+  if (!all.equal(data[, c(2)], gps_m$gps$w, tolerance = 0.00001)) {
+    stop(paste0("Provided GPS object and data object have different",
+                " exposure values."))
+  }
+
   # Expand the grid of parameters (alpha, beta, g_sigma) -----------------------
   tune_params <-  expand.grid(getElement(params, "alpha"),
                               getElement(params, "beta"),
                               getElement(params, "g_sigma"))
 
+  # Note: In the package, alpha is used to scale w and beta is used to scale
+  # GPS following the same convention in the method paper.
+
+
   if (nrow(tune_params) == 0) {
     stop(paste("Something went wrong with tuning parameters. ",
                "The expanded grid has not been generated."))
@@ -112,9 +138,9 @@ estimate_cerf_nngp <- function(data, w, GPS_m, params, formula,
 
   # Choose subset of tuning parameters based on tuning approach ----------------
 
-  if (getElement(params, "tune_app") == "all"){
+  if (getElement(params, "tune_app") == "all") {
     tune_params_subset <- tune_params
-  } else if (getElement(params, "tune_app") == "at_random"){
+  } else if (getElement(params, "tune_app") == "at_random") {
     stop("This approach is not implemented.")
   } else {
     stop(paste("The provided tune_app approach, ",
@@ -124,26 +150,22 @@ estimate_cerf_nngp <- function(data, w, GPS_m, params, formula,
 
   # Get other parameters -------------------------------------------------------
   n_neighbor <- getElement(params, "n_neighbor")
-  expand <- getElement(params, "expand")
   block_size <- getElement(params, "block_size")
 
   # Search for the best set of parameters --------------------------------------
-  design_mt <- as.data.frame(model.matrix(formula, data = data))
   optimal_cb_res <- find_optimal_nn(w_obs = data[, c(2)],
                                     w = w,
                                     y_obs = data[, c(1)],
-                                    GPS_m = GPS_m,
-                                    design_mt = design_mt,
+                                    gps_m = gps_m,
+                                    design_mt = data[, 3:ncol(data)],
                                     hyperparams = tune_params_subset,
                                     n_neighbor = n_neighbor,
                                     kernel_fn = kernel_fn,
-                                    expand = expand,
                                     block_size = block_size,
                                     nthread = nthread)
 
-
   # Extract the optimum hyperparameters
-  all_cb_res <- sapply(optimal_cb_res, '[[', 'cb')
+  all_cb_res <- sapply(optimal_cb_res, "[[", "cb")
   opt_idx_nn <- order(colMeans(abs(all_cb_res)))[1]
   posterior_mean <- optimal_cb_res[[opt_idx_nn]]$est
   nn_opt_param <- unlist(tune_params_subset[opt_idx_nn, ])
@@ -151,10 +173,10 @@ estimate_cerf_nngp <- function(data, w, GPS_m, params, formula,
   # Estimate noise -------------------------------------------------------------
   noise_nn <- estimate_noise_nn(hyperparam = nn_opt_param,
                                 w_obs = data[, c(2)],
-                                GPS_obs = GPS_m$GPS,
+                                GPS_obs = gps_m$gps$gps,
                                 y_obs = data[, c(1)],
                                 kernel_fn = kernel_fn,
-                                n_neighbor = n_neighbor * expand,
+                                n_neighbor = n_neighbor,
                                 nthread = nthread)
 
   # Compute posterior mean and standard deviation ------------------------------
@@ -163,10 +185,9 @@ estimate_cerf_nngp <- function(data, w, GPS_m, params, formula,
                                         w_obs = data[, c(2)],
                                         w = w,
                                         y_obs = data[, c(1)],
-                                        GPS_m = GPS_m,
+                                        gps_m = gps_m,
                                         kernel_fn = kernel_fn,
                                         n_neighbor = n_neighbor,
-                                        expand = expand,
                                         block_size = block_size,
                                         nthread = nthread)
 
@@ -176,17 +197,40 @@ estimate_cerf_nngp <- function(data, w, GPS_m, params, formula,
   logger::log_info("Done with estimating cerf using nngp approach ",
                    "Wall clock time: {t_nngp_2[[3]] - t_nngp_1[[3]]} s.")
 
+  # Compute covariate balance for original data --------------------------------
+  cov_balance_obj_org <- compute_w_corr(w = data[[2]],
+                                        covariate = data[, 3:ncol(data)],
+                                        weight = rep(1, nrow(data)))
+  cb_org <- cov_balance_obj_org$absolute_corr
 
-  # Build nngp_cerf S3 object
+
+  # Build nngp_cerf S3 object --------------------------------------------------
   result <- list()
   class(result) <- "cerf_nngp"
 
-  result$w <- w
+  # Hyper parameterss -------------------------
+  optimal_params <- nn_opt_param
+  names(optimal_params) <- c("alpha", "beta", "g_sigma")
+  result$optimal_params <- optimal_params
+  result$num_of_trial <- nrow(tune_params_subset)
+
+  # Data --------------------------------------
+  posterior <- list()
+  posterior$mean <- posterior_mean
+  posterior$sd <- posterior_sd
+  posterior$w <- w
+  result$posterior <- posterior
+
+  # Covariate balance -------------------------
   result$cb <- optimal_cb_res[[opt_idx_nn]]$cb
-  result$pst_mean <- posterior_mean
-  result$pst_sd <- posterior_sd
+  result$cb_org <- cb_org
+
+  # Function call -----------------------------
   result$fcall <- fcall
-  result$params <- nn_opt_param
+
+  t_nngp_2 <- proc.time()
+  logger::log_debug("Wall clock time to run estimate_cerf_nngp:",
+                    " {(t_nngp_2 -   t_nngp_1)[[3]]} seconds.")
 
   invisible(result)
 }
