@@ -5,14 +5,14 @@ test_that("estimate_noise_gp works as expected", {
 
 
   # Estimate GPS function
-  GPS_m <- train_gps(cov_mt = data[, -(1:2)],
-                     w_all = data$treat,
-                     sl_lib = c("SL.xgboost"),
-                     dnorm_log = FALSE)
+  gps_m <- estimate_gps(cov_mt = data[, -(1:2)],
+                        w_all = data$treat,
+                        sl_lib = c("SL.xgboost"),
+                        dnorm_log = FALSE)
 
-  GPS <- GPS_m$GPS
-  e_gps_pred <- GPS_m$e_gps_pred
-  e_gps_std <- GPS_m$e_gps_std
+  gps <- gps_m$gps$gps
+  e_gps_pred <- gps_m$gps$e_gps_pred
+  e_gps_std <- gps_m$gps$e_gps_std
 
   kernel_fn <- function(x) exp(-x ^ 2)
   hyperparam <- c(0.1, 0.2, 1)
@@ -23,7 +23,7 @@ test_that("estimate_noise_gp works as expected", {
 
   w_obs <- data[[2]]
 
-  scaled_obs <- cbind(w_obs * sqrt(1 / alpha), GPS * sqrt(1 / beta))
+  scaled_obs <- cbind(w_obs * sqrt(1 / alpha), gps * sqrt(1 / beta))
   sigma_obs <- g_sigma * kernel_fn(as.matrix(dist(scaled_obs))) +
                diag(nrow(scaled_obs))
   inv_sigma_obs <- compute_inverse(sigma_obs)
